<?php

/*************************************************************************

    XRef Analysis
    Copyright (C) 2007  Scott Auge and contributors 
    sauge@amduus.com scott_auge@yahoo.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
*************************************************************************/

/*
$Log: ldxref.php,v $
Revision 1.1  2007/05/22 06:40:49  sauge
Initial revision

*/
include_once ("os_tools.php");
include_once ("walk_propath.php");
include_once ("obj_db.php");
include_once ("parse_xref.php");
include_once ("random_string.php");

// Load in the arguments sent to the application from the ldxref
// script via environmental variables.

foreach ($_SERVER["argv"] as $V)
print ($V . "\n");

for ($i = 1; $i < $_SERVER["argc"]; $i++) {

  switch($_SERVER["argv"][$i]) {
    
    case "-appl":
      $ApplName = $_SERVER["argv"][++$i];
      break;

    case "-delete":
      $DoDelete = TRUE;
      break;
 
    case "-propath":
      $Propath = $_SERVER["argv"][++$i];
      break; 

  } // switch ()
} // for ()

// Let us know what is going on.  We also do a bit of validation.

print ("\n\n");
print (date(DATE_RFC822) . "\n");
print("APPL_NAME: $ApplName \n");
print("PROPATH: $Propath \n");
print("DO DELETE: $DoDelete \n");

if ($ApplName == "") {
  
  print ("Application Name Reqired... ABEND\n");
  exit(1);

}

if ($Propath == "") {

  print ("Propath Required... ABEND\n");
  exit (2);

}

// Go generate the list of files we are going to load in.  We
// send in the name of a file we will deposit the list into.

if (detectos() == "unix")
  $FileName = "/tmp/" . rand();
else
  $FileName = "c:\\temp\\" . rand();

walk_propath($FileName, $Propath);

// If we are deleting, then go ahead and delete

if ($DoDelete) {
  $SQL = "BEGIN WORK;DELETE FROM xref_raw WHERE app_name = '$ApplName';COMMIT WORK;";
  print ("Issuing: $SQL \n");
  $db->query($SQL);
  if (PEAR::isError($db)) {
    echo 'Standard Message: ' . $db->getMessage() . "\n";
    echo 'Standard Code: ' . $db->getCode() . "\n";
    echo 'DBMS/User Message: ' . $db->getUserInfo() . "\n";
    echo 'DBMS/Debug Message: ' . $db->getDebugInfo() . "\n";
    exit (3);
  } // if
}

// Use that list to populate our postgresql database.  See
// sql/README for more information about using other DBs.

load_xref_list ($FileName, $ApplName, $db);

// Housekeeping

$db->disconnect();

unlink ($FileName);

print ("Done\n");

//************************************************************************
// Interal Functions
//************************************************************************

function load_xref_list ($FileName, $ApplName, $db) {

  $FileHdl = fopen ($FileName, "r");

  while (! feof ($FileHdl)) {

    // For some reason, fgets() is giving us the line return also
    // so we need to prune that out.
    // TODO: Something different might be needed for windows

    $XRefFileName = fgets($FileHdl); 
    $XRefFileName = str_replace ("\n", "", $XRefFileName);
    if ($XRefFileName == "") break;

    print ("*******************************************************\n");
    print ("Loading: $XRefFileName .\n");

    load_xref_file ($XRefFileName, $ApplName, $db);

  } // while

  fclose ($FileHdl);

} // load_xref_list()

function load_xref_file ($FileName, $ApplName, $db) {

  $XRefFileHdl = fopen ($FileName, "r");

  while (! feof ($XRefFileHdl) ) {

    // We don't expect any lines larger than 4096!

    $Line = fgets ($XRefFileHdl);
    if ($Line == "") break;

    // Note fegts() seems to include the \n in the data
    $Line = str_replace ("\n", "", $Line); 
    $ParsedXRef = parse_xref($Line);

    // Warning: This SQL uses PostgreSQL escape functions

    $SQL = "BEGIN WORK;";

    $SQL .= "INSERT INTO xref_raw ("
         . "rec_id,"
         . " app_name,"
         . " procedure_name,"
         . " file_name,"
         . " line_number,"
         . " reference_type,"
         . " object_identifier) "
         . " VALUES ("
         . " '" . RandomString(20) . "', "
         . " '$ApplName', "
         . " '" . pg_escape_string($ParsedXRef["procedure_name"]) . "', "
         . " '" . pg_escape_string($ParsedXRef["file_name"]) . "', "
         . " '" . pg_escape_string($ParsedXRef["line_number"]) . "', "
         . " '" . pg_escape_string($ParsedXRef["reference_type"]) . "', "
         . " '" . pg_escape_string($ParsedXRef["object_identifier"]) . "'); ";

    $SQL .= "COMMIT WORK;";

    $db->query($SQL);

    if (PEAR::isError($db)) {
      echo 'Standard Message: ' . $db->getMessage() . "\n";
      echo 'Standard Code: ' . $db->getCode() . "\n";
      echo 'DBMS/User Message: ' . $db->getUserInfo() . "\n";
      echo 'DBMS/Debug Message: ' . $db->getDebugInfo() . "\n";
      exit (3);
    } // if
   
    print ("Issuing: $SQL \n"); 

  } // while

  fclose ($XRefFileHdl);

} // load_xref_file ()

?>
